﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

//System
using System.Threading;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

//EMGU
using Emgu.CV;
using Emgu.CV.Structure;
using Emgu.Util;

//Microsoft Kinect SDK
//These will need recompiling and replacing if your using x86
using Microsoft.Kinect;
using Microsoft.Kinect.Toolkit;

namespace Capture_Kinect
{
    public partial class Form1 : Form
    {
        KinectSensor KS; //Kinect Sensor

        ColorImageFrame CF; //Variable for our colour frame
        DepthImageFrame DF; //variable for our depth frame

        Thread KinectBuffer; //Thread for processing Data from frames

        
        public Form1()
        {

            InitializeComponent();

            //// Look through all sensors and start the first connected one.
            //// This requires that a Kinect is connected at the time of app startup.
            //// To make your app robust against plug/unplug, 
            //// it is recommended to use KinectSensorChooser provided in Microsoft.Kinect.Toolkit (See components in Toolkit Browser).
            foreach (var potentialSensor in KinectSensor.KinectSensors)
            {
                if (potentialSensor.Status == KinectStatus.Connected)
                {
                    this.KS = potentialSensor;
                    break;
                }
            }

            //If we have a Kinect Sensor we will set it up
            if (null != KS)
            {
                // Turn on the color stream to receive color frames
                KS.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30);
                //Turn on the depth stream to recieve depth frames
                KS.DepthStream.Enable(DepthImageFormat.Resolution640x480Fps30);
                
                //Start the Streaming process
                KS.Start();
                //Create a link to a callback to deal with the frames
                KS.AllFramesReady += new EventHandler<AllFramesReadyEventArgs>(KS_AllFramesReady);

                //We set up a thread to process the image/disparty map from the kinect
                //Why? The kinect AllFramesReady has a timeout if it has not finished the streams will simply stop
                KinectBuffer = new Thread(ProcessBuffer);

                //Let's play with the motor as well just because we can
                KS.ElevationAngle = 0;
                Thread.Sleep(10);
                KS.ElevationAngle = 24;
                Thread.Sleep(100);
                KS.ElevationAngle = 12;

            }
          
        }


        void ProcessBuffer()
        {
            //This takes the byte[] array from the kinect and makes a bitmap from the colour data for us
            byte[] pixeldata = new byte[CF.PixelDataLength];
            CF.CopyPixelDataTo(pixeldata);
            Bitmap bmap = new Bitmap(CF.Width, CF.Height, PixelFormat.Format32bppRgb);
            BitmapData bmapdata = bmap.LockBits(new Rectangle(0, 0, CF.Width, CF.Height), ImageLockMode.WriteOnly, bmap.PixelFormat);
            IntPtr ptr = bmapdata.Scan0;
            Marshal.Copy(pixeldata, 0, ptr, CF.PixelDataLength);
            bmap.UnlockBits(bmapdata);
            //display our colour frame
            Image<Bgr, byte> img = new Image<Bgr, Byte>(bmap);
            ImageView.Image = img.ToBitmap();

            //This takes the short[] array from the kinect and makes a bitmap from the depth data for us
            short[] disparitydata = new short[DF.PixelDataLength];
            DF.CopyPixelDataTo(disparitydata);
            bmap = new Bitmap(DF.Width, DF.Height, PixelFormat.Format16bppRgb555);
            bmapdata = bmap.LockBits(new Rectangle(0, 0, DF.Width, DF.Height), ImageLockMode.WriteOnly, bmap.PixelFormat);
            ptr = bmapdata.Scan0;
            Marshal.Copy(disparitydata, 0, ptr, DF.PixelDataLength);
            bmap.UnlockBits(bmapdata);

            //Here we play with the data a bit we use the pointer as passing the bitmap fails to work.
            Image<Gray, ushort> depth = new Image<Gray, ushort>(640, 480, 640 * 2, ptr);
            depth *= ushort.MaxValue / 20000; //normilsation

            //From here we play with what we've got
            Image<Gray, double> depthd = depth.Convert<Gray, double>(); //conver to double
            //Lets make it a colured depth image rather than black and white
            using (Image<Bgr, double> _img = new Image<Bgr, double>(640, 480))
            {
                //Lets invert it so the closer objects are bright _Not() will not work so
                depthd *= -1;
                depthd += 65535;

                //Copy the depth data over to our coloured array
                _img[0] = depthd.Copy();
                _img[1] = (depthd * 2).Copy();
                _img[2] = depthd.Copy();

                DepthView.Image = _img.ToBitmap(); //Display it
            }
        }

        void KS_AllFramesReady(object sender, AllFramesReadyEventArgs e)
        {
            //Aquire the data for the frames 
            CF = e.OpenColorImageFrame(); //colour image
            DF = e.OpenDepthImageFrame(); //disparity map

            //Since we are using both frames within our thread end this one if either are null
            if (CF == null || DF == null) return;

            //Start a thread to deal with frame data
            //a new thread prevents timeout issues in this callback
            if (KinectBuffer.ThreadState != ThreadState.Running)
            {
                KinectBuffer = new Thread(ProcessBuffer);
                KinectBuffer.Start();
            }
        }

    }
}
